@@ -2,19 +2,19 @@
 Rollouts - A high-quality Python package for generating multiple LLM responses.
 """
 
-from .client import RolloutsClient, create_client
+from .client import RolloutsClient
 from .datatypes import Rollouts, Response, Usage
-from .config import Config
 from .openrouter import OpenRouter
+from .migrate import migrate_cache, auto_migrate
 
-__version__ = "0.1.0"
+__version__ = "0.1.4"
 
 __all__ = [
     "RolloutsClient",
-    "create_client",
     "Rollouts",
     "Response",
     "Usage",
-    "Config",
     "OpenRouter",
-]
\ No newline at end of file
+    "migrate_cache",
+    "auto_migrate",
+]
