@@ -52,7 +52,7 @@ class RolloutsClient:
         max_retries: int = 100,
         timeout: int = 300,
         verbose: bool = False,
-        use_cache: Union[bool, str] = "sql",
+        use_cache: Union[bool, str] = "json",
         cache_dir: str = ".rollouts",
         requests_per_minute: Optional[int] = None,
         progress_bar: bool = True,
@@ -220,7 +220,7 @@ class RolloutsClient:
 
     async def agenerate(
         self,
-        prompt: str,
+        prompt: Union[str, List[dict]],
         n_samples: Optional[int] = None,
         temperature: Optional[float] = None,
         top_p: Optional[float] = None,
@@ -473,7 +473,7 @@ class RolloutsClient:
 
     def generate(
         self,
-        prompt: str,
+        prompt: Union[str, List[dict]],
         n_samples: Optional[int] = None,
         progress_bar: Optional[bool] = None,
         **kwargs,
